@@ -1,7 +1,10 @@
 import os
+import sys
 import json
+import shelve
 import pandas as pd
 from time import sleep
+from tempfile import gettempdir
 from urllib import parse, request
 from datetime import datetime
 from functools import partial
@@ -24,13 +27,13 @@ class DataClient:
 
     base_url = "https://service-an8w6tgn-1253762454.sh.apigw.tencentcs.com"
 
-    def __init__(self, token="", debug=False):
+    def __init__(self, token="", debug=False, use_cache=False):
         self.debug = debug
         self.token = os.environ["DC_TOKEN"] if "DC_TOKEN" in os.environ else token
 
-        self.get_bars = partial(self.history, data_type="bar", freq="QS")
-        self.get_ticks = partial(self.history, data_type="tick", freq="12BH")
-        self.get_dailys = partial(self.history, data_type="daily", freq="YS")
+        self.get_bars = partial(self.history, data_type="bar", freq="QS", use_cache=use_cache)
+        self.get_ticks = partial(self.history, data_type="tick", freq="12BH", use_cache=use_cache)
+        self.get_dailys = partial(self.history, data_type="daily", freq="YS", use_cache=use_cache)
 
     def status_check(self):
         while True:
@@ -45,7 +48,7 @@ class DataClient:
                 print(e, "retrying...")
                 sleep(1)
 
-    def history(self, symbol, start_data, end_date, data_type, freq):
+    def history(self, symbol, start_data, end_date, data_type, freq, use_cache):
         date_split = pd.date_range(start_data, end_date, freq=freq)
         date_split = [start_data, *date_split, end_date]
         data = pd.DataFrame()
@@ -53,17 +56,28 @@ class DataClient:
         for index, _ in enumerate(date_split):
             if index + 1 != len(date_split):
                 args = {
-                    "token": self.token,
                     "symbol": symbol,
                     "data_type": data_type,
                     "start_date": datetime.strftime(date_split[index], "%Y%m%d%H%M%S"),
                     "end_date": datetime.strftime(date_split[index+1], "%Y%m%d%H%M%S"),
                 }
+                key = json.dumps(args, sort_keys=True)
+                if sys.platform == "darwin":
+                    cache_client = {}
+                else:
+                    cache_file = os.path.join(gettempdir(), key)
+                    cache_client = shelve.open(cache_file, writeback=True)
                 try:
+                    if use_cache:
+                        if key in cache_client:
+                            data = data.append(cache_client[key])
+                            continue
                     df = pd.read_json(
-                        self.base_url + "/md?" + parse.urlencode(args), orient="split"
+                        self.base_url + "/md?" + parse.urlencode({ "token": self.token, **args }), orient="split"
                     ).set_index("datetime")
                     data = data.append(df)
+                    # save to cache
+                    cache_client[key] = df
                     if self.debug:
                         print("下载完成", date_split[index], "~", date_split[index+1])
                 except:
@@ -73,7 +87,7 @@ class DataClient:
 
 
 if __name__ == "__main__":
-    client = DataClient("your token!")
+    client = DataClient("your token!", use_cache=True)
     # 日线
     dailys = client.get_dailys(
         "CFFEX.IF@INDEX",
@@ -84,15 +98,15 @@ if __name__ == "__main__":
     # 分钟
     bars = client.get_bars(
         "SHFE.rb@INDEX",
-        start_data=datetime(2021, 5, 6, 21, 0, 0),
+        start_data=datetime(2020, 4, 6, 21, 0, 0),
         end_date=datetime(2021, 6, 4, 15, 0, 0)
     )
     print(bars)
 
-    # tick
-    ticks = client.get_ticks(
-        "SHFE.rb2110",
-        start_data=datetime(2021, 6, 3, 21, 0, 0),
-        end_date=datetime(2021, 6, 4, 15, 0, 0)
-    )
-    print(ticks)
+    # # tick
+    # ticks = client.get_ticks(
+    #     "SHFE.rb2110",
+    #     start_data=datetime(2021, 6, 3, 21, 0, 0),
+    #     end_date=datetime(2021, 6, 4, 15, 0, 0)
+    # )
+    # print(ticks)
